/*-
 * Copyright (c) 2004 - 2008 CTPP Team
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the CTPP Team nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      CTPPException.hpp
 *
 * $CTPP$
 */
#ifndef _CTPP_EXCEPTION_HPP__
#define _CTPP_EXCEPTION_HPP__ 1

#include <CTPP2Types.h>

#include <string.h>
#include <stdlib.h>

#ifdef _USE_STL_AS_BASE_EXCEPTION
#include <exception>
#endif

namespace CTPP // C++ Template Engine
{

/**
  @class CTPPException CTPPException.hpp <CTPPException.hpp>
  @brief Base class for all CAS exceptions.
*/
class CTPPException
#ifdef _USE_STL_AS_BASE_EXCEPTION
:
  public std::exception
#endif // _USE_STL_AS_BASE_EXCEPTION
{
public:
	/** @return A asciz string describing the general cause of error */
	virtual CCHAR_P what() const throw() { return "Unknown Error"; }
	/** @brief A virtual destructor */
	virtual ~CTPPException() throw() { ;; }
};

/**
  @class CTPPLogicError CTPPException.hpp <CTPPException.hpp>
  @brief Logic error
*/
class CTPPLogicError:
  public CTPPException
{
public:
	/**
	  @brief Constrcutor
	  @param sIReason - Error description
	*/
	inline CTPPLogicError(CCHAR_P sIReason) throw(): sReason(strdup(sIReason)) { ;; }

	/** @return A asciz string describing the general cause of error */
	inline CCHAR_P what() const throw() { return sReason; }

	/** A destructor */
	inline ~CTPPLogicError() throw() { free(sReason); }
private:
	/** Error description */
	CHAR_P sReason;
};

/**
  @class CDTTypeCastException CTPPException.hpp <CTPPException.hpp>
  @brief Invalid type cast
*/
class CDTTypeCastException:
  public CTPPException
{
public:
	/**
	  @brief Constrcutor
	  @param sIReason - Error description
	*/
	inline CDTTypeCastException(CCHAR_P sIReason) throw(): sReason(strdup(sIReason)) { ;; }

	/** @return A asciz string describing the general cause of error */
	inline CCHAR_P what() const throw() { return sReason; }

	/** A destructor */
	inline ~CDTTypeCastException() throw() { free(sReason); }
private:
	/** Error description */
	CHAR_P sReason;
};

/**
  @class CDTAccessException CTPPException.hpp <CTPPException.hpp>
  @brief Access violation exception
*/
class CDTAccessException:
  public CTPPException
{
public:

	/** @return A asciz string describing the general cause of error */
	inline CCHAR_P what() const throw() { return "operator []"; }

	/** A destructor */
	inline ~CDTAccessException() throw() { ;; }
private:

};

/**
  @class CDTRangeException CTPPException.hpp <CTPPException.hpp>
  @brief Array index is out of bounds
*/
class CDTRangeException:
  public CTPPException
{
public:

	/** @return A asciz string describing the general cause of error */
	inline CCHAR_P what() const throw() { return "pos > size"; }

	/** A destructor */
	inline ~CDTRangeException() throw() { ;; }
private:

};

/**
  @class CTPPUnixException CTPPException.hpp <CTPPException.hpp>
  @brief Unix error code
*/
class CTPPUnixException:
  public CTPPException
{
public:
	/**
	  @brief Constrcutor
	  @param sIReason - Error description
	  @param iIErrno - Unix error code
	*/
	inline CTPPUnixException(CCHAR_P sIReason, INT_32 iIErrno) throw(): sReason(strdup(sIReason)), iErrno(iIErrno) { ;; }

	/** @return A asciz string describing the general cause of error */
	inline CCHAR_P what() const throw() { return sReason; }

	/** @return error code */
	inline INT_32 ErrNo() const throw() { return iErrno; }

	/** A destructor */
	inline ~CTPPUnixException() throw() { free(sReason); }
private:
	/** Error description */
	CHAR_P sReason;
	/** Error code        */
	INT_32 iErrno;
};

/**
  @class CTPPNullPointerException CTPPException.hpp <CTPPException.hpp>
  @brief Pointer is NULL
*/
class CTPPNullPointerException:
  public CTPPException
{
public:
	/**
	  @brief Constrcutor
	  @param sIReason - Error description
	  @param iIErrno - Unix error code
	*/
	inline CTPPNullPointerException(CCHAR_P sIReason) throw(): sReason(strdup(sIReason)) { ;; }

	/** @return A asciz string describing the general cause of error */
	inline CCHAR_P what() const throw() { return sReason; }

	/** A destructor */
	inline ~CTPPNullPointerException() throw() { free(sReason); }
private:
	/** Error description */
	CHAR_P sReason;
};

} // namespace CTPP
#endif // _CTPP_EXCEPTION_HPP__
// End.
