/*-
 * Copyright (c) 2004 - 2008 CTPP Team
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the CTPP Team nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      VMSTDLib.hpp
 *
 * $CTPP$
 */
#ifndef _VM_STDLIB_HPP__
#define _VM_STDLIB_HPP__ 1

#include <VMSyscall.hpp>

#ifdef ICONV_SUPPORT

#include <iconv.h>
#include <map>
#include <string>

#endif // ICONV_SUPPORT

/**
  @file VMSTDLib.hpp
  @brief Virtual machine standard library
*/

namespace CTPP // C++ Template Engine
{

//
// Escape value
//
CDT URLEscape(const CDT & oCDT);

//
// Escape value
//
CDT HTMLEscape(const CDT & oCDT);

//
// Escape value
//
CDT XMLEscape(const CDT & oCDT);

// FWD
class SyscallFactory;

/**
  @class STDLibInitializer VMSTDLib.hpp <VMSTDLib.hpp>
  @brief List of standard functions & initializer for standard library
*/
class STDLibInitializer
{
public:
	/**
	  @brief Get list of available functions
	  @return NULL-rerminated list
	*/
	static CCHAR_P * GetFnList();

	/**
	  @brief Initialize all functions in library
	  @param oSyscallFactory - object factory
	*/
	static void InitLibrary(SyscallFactory & oSyscallFactory);

	/**
	  @brief Destroy all functions in library
	  @param oSyscallFactory - object factory
	*/
	static void DestroyLibrary(SyscallFactory & oSyscallFactory);

	/**
	  @brief Create handler
	  @param szHandler - function name
	  @return new object of CTPP2 function
	*/
	static SyscallHandler * CreateHandler(CCHAR_P szHandler);

	/**
	  @brief Create handler
	  @param pHandler - function object
	  @return new object of CTPP2 function
	*/
	static void DestroyHandler(SyscallHandler * pHandler);
private:

	/** List of functions */
	static CCHAR_P aSTDFNList[];
};

/**
  @class FnEmitter VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Emit CDT variables into virual machine
*/
class FnEmitter:
  public SyscallHandler
{
public:
	/**
	  @brief A destructor
	*/
	~FnEmitter() throw();

private:
	friend class STDLibInitializer;

	/** Global CDT object */
	CDT     * pCDT;

	/**
	  @brief Constructor
	*/
	FnEmitter();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnEmitter(const FnEmitter & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnEmitter & operator =(const FnEmitter & oRhs);

	/**
	  @brief Pre-execution handler setup
	  @param oCollector - output data collector
	  @param oCDT - CTPP2 parameters
	  @param oSyscalls - Syscalls segment
	  @param oStaticData - Static data segment
	  @param oStaticText - Static text segment
	  @return 0 - if success, -1 - if any error occured
	*/
	INT_32 PreExecuteSetup(OutputCollector    & oCollector,
	                       CDT                & oCDT,
	                       const ReducedStaticText  & oSyscalls,
	                       const ReducedStaticData  & oStaticData,
	                       const ReducedStaticText  & oStaticText);
	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

/**
  @class FnOutput VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Construct output of value
*/
class FnOutput:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnOutput() throw();

private:
	friend class STDLibInitializer;


	/** Ouput data collector  */
	OutputCollector * pCollector;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnOutput();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnOutput(const FnOutput & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnOutput & operator =(const FnOutput & oRhs);

	/**
	  @brief Pre-execution handler setup
	  @param oCollector - output data collector
	  @param oCDT - CTPP2 parameters
	  @param oSyscalls - Syscalls segment
	  @param oStaticData - Static data segment
	  @param oStaticText - Static text segment
	  @return 0 - if success, -1 - if any error occured
	*/
	INT_32 PreExecuteSetup(OutputCollector    & oCollector,
	                       CDT                & oCDT,
	                       const ReducedStaticText  & oSyscalls,
	                       const ReducedStaticData  & oStaticData,
	                       const ReducedStaticText  & oStaticText);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

/**
  @class FnOutputText VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Construct output of stored text
*/
class FnOutputText:
  public SyscallHandler
{
	/**
	  @brief Constructor
	  @param pICollector - output data collector
	  @param pIStaticText - CTPP static text segment
	*/
 	FnOutputText(OutputCollector * pICollector, const ReducedStaticText * pIStaticText);

	/**
	  @brief A destructor
	*/
	~FnOutputText() throw();

private:
	friend class STDLibInitializer;

	/** Ouput data collector     */
	OutputCollector          * pCollector;
	/** CTPP static text segment */
	const ReducedStaticText  * pStaticText;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnOutputText();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnOutputText(const FnOutputText & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnOutputText & operator =(const FnOutputText & oRhs);

	/**
	  @brief Pre-execution handler setup
	  @param oCollector - output data collector
	  @param oCDT - CTPP2 parameters
	  @param oSyscalls - Syscalls segment
	  @param oStaticData - Static data segment
	  @param oStaticText - Static text segment
	  @return 0 - if success, -1 - if any error occured
	*/
	INT_32 PreExecuteSetup(OutputCollector    & oCollector,
	                       CDT                & oCDT,
	                       const ReducedStaticText  & oSyscalls,
	                       const ReducedStaticData  & oStaticData,
	                       const ReducedStaticText  & oStaticText);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

/**
  @class FnOutputInt VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Construct output of stored integer value
*/
class FnOutputInt:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnOutputInt() throw();

private:
	friend class STDLibInitializer;

	/** Ouput data collector     */
	OutputCollector          * pCollector;
	/** CTPP static text segment */
	const ReducedStaticData  * pStaticData;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnOutputInt();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnOutputInt(const FnOutputInt & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnOutputInt & operator =(const FnOutputInt & oRhs);

	/**
	  @brief Pre-execution handler setup
	  @param oCollector - output data collector
	  @param oCDT - CTPP2 parameters
	  @param oSyscalls - Syscalls segment
	  @param oStaticData - Static data segment
	  @param oStaticText - Static text segment
	  @return 0 - if success, -1 - if any error occured
	*/
	INT_32 PreExecuteSetup(OutputCollector    & oCollector,
	                       CDT                & oCDT,
	                       const ReducedStaticText  & oSyscalls,
	                       const ReducedStaticData  & oStaticData,
	                       const ReducedStaticText  & oStaticText);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

/**
  @class FnOutputFloat VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Construct output of stored float value
*/
class FnOutputFloat:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnOutputFloat() throw();

private:
	friend class STDLibInitializer;

	/** Ouput data collector     */
	OutputCollector          * pCollector;
	/** CTPP static text segment */
	const ReducedStaticData  * pStaticData;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnOutputFloat();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnOutputFloat(const FnOutputFloat & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnOutputFloat & operator =(const FnOutputFloat & oRhs);

	/**
	  @brief Pre-execution handler setup
	  @param oCollector - output data collector
	  @param oCDT - CTPP2 parameters
	  @param oSyscalls - Syscalls segment
	  @param oStaticData - Static data segment
	  @param oStaticText - Static text segment
	  @return 0 - if success, -1 - if any error occured
	*/
	INT_32 PreExecuteSetup(OutputCollector    & oCollector,
	                       CDT                & oCDT,
	                       const ReducedStaticText  & oSyscalls,
	                       const ReducedStaticData  & oStaticData,
	                       const ReducedStaticText  & oStaticText);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};


/**
  @class FnVersion VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Get version of CTPP2 and CTPP2 standard library
*/
class FnVersion:
  public SyscallHandler
{
	/**
	  @brief Constructor
	*/
 	FnVersion();

	/**
	  @brief A destructor
	*/
	~FnVersion() throw();

private:
	friend class STDLibInitializer;

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

/**
  @class FnURLEscape VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Escape value, see http://www.ietf.org/rfc/rfc2396.txt
*/
class FnURLEscape:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnURLEscape() throw();

private:
	friend class STDLibInitializer;


	/** Ouput data collector  */
	OutputCollector * pCollector;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnURLEscape();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnURLEscape(const FnURLEscape & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnURLEscape & operator =(const FnURLEscape & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

/**
  @class FnHTMLEscape VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Escape value, see http://www.w3.org/
*/
class FnHTMLEscape:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnHTMLEscape() throw();

private:
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnHTMLEscape();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnHTMLEscape(const FnHTMLEscape & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnHTMLEscape & operator =(const FnHTMLEscape & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

/**
  @class FnXMLEscape VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Escape value, see http://xml.w3.org/
*/
class FnXMLEscape:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnXMLEscape() throw();

private:
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnXMLEscape();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnXMLEscape(const FnHTMLEscape & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnXMLEscape & operator =(const FnXMLEscape & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

#ifdef GETTEXT_SUPPORT

class FnGetText_;

/**
  @class FnGetText VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Localization support
*/
class FnGetText:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnGetText() throw();

private:
	friend class FnGetText_;
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnGetText();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnGetText(const FnGetText & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnGetText & operator =(const FnGetText & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	virtual CCHAR_P GetName();
};

/**
  @class FnGetText_ VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Escape value, see http://xml.w3.org/
*/
class FnGetText_:
  public FnGetText
{
	/**
	  @brief A destructor
	*/
	~FnGetText_() throw();

private:
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnGetText_();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnGetText_(const FnHTMLEscape & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnGetText_ & operator =(const FnGetText_ & oRhs);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};
#endif // GETTEXT_SUPPORT

/**
  @class FnInSet VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Localization support
*/
class FnInSet:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnInSet() throw();

private:
	friend class FnInSet_;
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnInSet();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnInSet(const FnInSet & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnInSet & operator =(const FnInSet & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	virtual CCHAR_P GetName();
};

/**
  @class FnHrefParam VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Localization support
*/
class FnHrefParam:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnHrefParam() throw();

private:
	friend class FnHrefParam_;
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnHrefParam();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnHrefParam(const FnHrefParam & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnHrefParam & operator =(const FnHrefParam & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	virtual CCHAR_P GetName();
};

/**
  @class FnFormParam VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Localization support
*/
class FnFormParam:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnFormParam() throw();

private:
	friend class FnFormParam_;
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnFormParam();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnFormParam(const FnFormParam & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnFormParam & operator =(const FnFormParam & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	virtual CCHAR_P GetName();
};

/**
  @class FnDateFormat VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Date formatting function
*/
class FnDateFormat:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnDateFormat() throw();

private:
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnDateFormat();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnDateFormat(const FnDateFormat & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnDateFormat & operator =(const FnDateFormat & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

/**
  @class FnBase64Encode VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Base64 encoder
*/
class FnBase64Encode:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnBase64Encode() throw();

private:
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnBase64Encode();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnBase64Encode(const FnBase64Encode & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnBase64Encode & operator =(const FnBase64Encode & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

/**
  @class FnBase64Decode VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Base64 decoder
*/
class FnBase64Decode:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnBase64Decode() throw();

private:
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnBase64Decode();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnBase64Decode(const FnBase64Decode & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnBase64Decode & operator =(const FnBase64Decode & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

#ifdef MD5_SUPPORT
/**
  @class FnMD5 VMSTDLib.hpp <VMSTDLib.hpp>
  @brief MD5 Hash support
*/
class FnMD5:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnMD5() throw();

private:
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnMD5();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnMD5(const FnMD5 & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnMD5 & operator =(const FnMD5 & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};
#endif // MD5_SUPPORT

#ifdef ICONV_SUPPORT
/**
  @class FnIconv VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Iconv converter
*/
class FnIconv:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnIconv() throw();

private:
	friend class STDLibInitializer;

	std::map<std::string, iconv_t> mIconvMap;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnIconv();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnIconv(const FnIconv & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnIconv & operator =(const FnIconv & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};
#endif // ICONV_SUPPORT

/**
  @class FnObjDump VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Dump variables
*/
class FnObjDump:
  public SyscallHandler
{
public:
	/**
	  @brief A destructor
	*/
	~FnObjDump() throw();

private:
	friend class STDLibInitializer;

	/** Global CDT object */
	CDT     * pCDT;

	/**
	  @brief Constructor
	*/
	FnObjDump();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnObjDump(const FnObjDump & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnObjDump & operator =(const FnObjDump & oRhs);

	/**
	  @brief Pre-execution handler setup
	  @param oCollector - output data collector
	  @param oCDT - CTPP2 parameters
	  @param oSyscalls - Syscalls segment
	  @param oStaticData - Static data segment
	  @param oStaticText - Static text segment
	  @return 0 - if success, -1 - if any error occured
	*/
	INT_32 PreExecuteSetup(OutputCollector    & oCollector,
	                       CDT                & oCDT,
	                       const ReducedStaticText  & oSyscalls,
	                       const ReducedStaticData  & oStaticData,
	                       const ReducedStaticText  & oStaticText);
	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

/**
  @class FnCast VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Cast to specified type
*/
class FnCast:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnCast() throw();

private:
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnCast();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnCast(const FnCast & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnCast & operator =(const FnCast & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

/**
  @class FnDefined VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Check UNDEF type
*/
class FnDefined:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnDefined() throw();

private:
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnDefined();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnDefined(const FnDefined & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnDefined & operator =(const FnDefined & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

/**
  @class FnSize VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Get array/hash size
*/
class FnSize:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnSize() throw();

private:
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnSize();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnSize(const FnSize & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnSize & operator =(const FnSize & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

/**
  @class FnNumFormat VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Format number
*/
class FnNumFormat:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnNumFormat() throw();

private:
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnNumFormat();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnNumFormat(const FnNumFormat & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnNumFormat & operator =(const FnNumFormat & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

/**
  @class FnError VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Permanent error (only for VM testing)
*/
class FnError:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnError() throw();

private:
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnError();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnError(const FnError & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnError & operator =(const FnError & oRhs);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

/**
  @class FnBBCode VMSTDLib.hpp <VMSTDLib.hpp>
  @brief Construct output of value
*/
class FnBBCode:
  public SyscallHandler
{
	/**
	  @brief A destructor
	*/
	~FnBBCode() throw();

private:
	friend class STDLibInitializer;

	// ///////////////////////////////////////////////////////////////

	/**
	  @brief Constructor
	*/
	FnBBCode();

	/**
	  @brief Copy constructor
	  @param oRhs - object to copy
	*/
	FnBBCode(const FnBBCode & oRhs);

	/**
	  @brief Copy operator =
	  @param oRhs - object to copy
	*/
	FnBBCode & operator =(const FnBBCode & oRhs);

	/**
	  @brief Pre-execution handler setup
	  @param oCollector - output data collector
	  @param oCDT - CTPP2 parameters
	  @param oSyscalls - Syscalls segment
	  @param oStaticData - Static data segment
	  @param oStaticText - Static text segment
	  @return 0 - if success, -1 - if any error occured
	*/
	INT_32 PreExecuteSetup(OutputCollector    & oCollector,
	                       CDT                & oCDT,
	                       const ReducedStaticText  & oSyscalls,
	                       const ReducedStaticData  & oStaticData,
	                       const ReducedStaticText  & oStaticText);

	/**
	  @brief Handler
	  @param aArguments - list of arguments
	  @param iArgNum - number of arguments
	  @param oCDTRetVal - return value
	*/
	INT_32 Handler(CDT * aArguments, const UINT_32 & iArgNum, CDT & oCDTRetVal);

	/**
	  @brief Get function name
	*/
	CCHAR_P GetName();
};

} // namespace CTPP
#endif // _VM_SYSCALL_HPP__
// End.
